//	HyperbolicHelper Script is a Hyperbolic Poincaré polygon tessellation helper Script by Brian McMillin March 11-22, 2014.
//	This script assists in the creation of central hyperbolic polygons in the Poincaré disk.
//	The center of the Poincaré disk is selected first, but is not used as a center of inversion.
//	The script calculates another external point, centerInvPt, which can be used after the script terminates,
//  as a center of inversion, to create additional polygons.  Any polygon(s) may be re-replicated, with inversion,
//	by the user, with centers of inversion located with the Inversion3pt script.
//	The mathematical nomenclature for the tessellation is {p,q}, where p is the number of sides of a polygon, (sideCt),
//	and q is the number of polygons which meet at each polygon vertex, (vertexCt).
//	Note that a p-sided polygon may be subdivided into p regions, or 2*p hyperbolic triangles, etcetera.
//	Also note that there are some constraints on the minimal values for p and q, as per the table in reference #1.
//	p and q are positive integers, and (p - 2)(q - 2) > 4.
//	Lower combinations of p and q do not work for this script.  (They only work for planes and spheres.)
//	Reference #1:  http://www.d.umn.edu/~ddunham/md07tlk.pdf Creating Regular Repeating Hyperbolic Patterns.
//	https://cms.math.ca/cmb/v40/coxeter6871.pdf The trigonometry.
//	http://moniker.name/worldmaking/?p=385 Hyperbolic Geometry: Poincaré Disc Tessellation How-to.
//	http://citeseerx.ist.psu.edu/viewdoc/download?doi=10.1.1.133.8746&rep=rep1&type=pdf Compass and Straightedge in the Poincaré Disk.
//	Much code from LineWeb, RingCircle, ChainMaker and BisectorLine was used.
//	Note that the side of the polygon is straight in Hyperbolic space, but its projection in euclidean space is a circular arc.
//	Also note that all of the inverted polygons are the same size in hyperbolic space, even though they appear different in
//	euclidean space.  Also, angles are preserved.

#include "GetPoint.js"
#include "GetObjects.js"

function alert( msg )
{
	moi.ui.commandUI.alert( msg );
}

function Cancel( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].cancel();
		
	factories.length = 0;
}

function Commit( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].commit();
}

function dotproduct( v0, v1 )
{
	return (v0.x * v1.x) + (v0.y * v1.y) + (v0.z * v1.z);
}

function InvertPt( centerPt, evalPt, pR )
{
	var vm = moi.vectorMath;
	var dirOP = vm.makeVector(centerPt, evalPt);  // to be vector direction.
	var lensqOP = lengthsq( dirOP );  // length squared.
	var pRsq = pR * pR;  //radius squared.
	var mult = pRsq / lensqOP;  //combine the math.
	return vm.createPoint( centerPt.x + dirOP.x * mult, centerPt.y + dirOP.y *mult, centerPt.z + dirOP.z * mult );	
}

function lengthsq( v )
{
	return dotproduct( v, v );
}

function Update( centerPt,sideCt, vertexCt, numpoints, disk, diskLine, circleInv, rotate1Line, rotate2Line, intersect1, intersect2, arcSide, poly )
{
	// Test if (p,q) combination is less than minimum.
	// (p-2)*(q-2) must be greater than 4.
	if ((sideCt-2)*(vertexCt-2) <= 4)
	{
		alert ("(p,q) combination is less than minimum, so increase sideCt and/or vertexCt")
		return;
	}

	var gd = moi.geometryDatabase;
	
//	Cancel the arrays of factories:
	Cancel( disk );
	Cancel( diskLine );
	Cancel( circleInv );
	Cancel( rotate1Line );
	Cancel( rotate2Line );
	Cancel( intersect1 );
	Cancel( intersect2 );	
	Cancel( arcSide );
	Cancel( poly );

//	Show the Poincare disk unit circle in top view.
	var diskfactory = moi.command.createFactory( 'circle' );
	var FrameOrigin = centerPt;
	var FrameXAxis = moi.vectorMath.createPoint( 1, 0, 0 );
	var FrameYAxis = moi.vectorMath.createPoint(0, 1, 0 );
	var Frame = moi.vectorMath.createFrame( FrameOrigin, FrameXAxis, FrameYAxis );
	// Set the frame on the factory.
	diskfactory.setInput( 1, Frame ); //used 3 more times below.
	// Set the flag to tell the factory to use radius.
	diskfactory.setInput( 0, true );
	// Set the radius = 1.0, on the factory and update it to generate the initial output.
	diskfactory.setInput( 3, 1.0 );
	diskfactory.update();	
	
	// Calculate the center of inversion, which is outside of the Poincaré disk.  (Coxeter paper & moniker site.)
	var s = Math.sin( Math.PI / sideCt ); //p
	var c = Math.cos( Math.PI / vertexCt ); //q
	var radiusInv = 1 / Math.sqrt((( c * c ) / ( s * s)) - 1); //r
	var centerDist = 1 / Math.sqrt(1 - (( s * s ) / (c * c ))); //d
	var centerInvPt = moi.vectorMath.createPoint( centerPt.x + centerDist, centerPt.y, centerPt.z );
	var diskradiusPt = moi.vectorMath.createPoint( centerPt.x + 1.0, centerPt.y, centerPt.z ); //point on poincare unit disk.
		
	// Create a line of length 1.0, from centerPt towards centerInvPt.
	var diskLinefactory = moi.command.createFactory( 'line' );
	diskLinefactory.setInput( 0, centerPt );
	diskLinefactory.setInput( 1, diskradiusPt );
	diskLinefactory.update();
	var diskLineObj = diskLinefactory.getCreatedObjects();
//	var diskLine = diskLineObj.item(0); Not needed.

//	Show invCircle, the circle of inversion, in top view.
	var circleInvfactory = moi.command.createFactory( 'circle' );
	var FrameOrigin = centerInvPt;
	var Frame2 = moi.vectorMath.createFrame( FrameOrigin, FrameXAxis, FrameYAxis );
	circleInvfactory.setInput( 1, Frame2 );
	circleInvfactory.setInput( 0, true );
	circleInvfactory.setInput( 3, radiusInv );
	circleInvfactory.update();  
	var invCircleObj = circleInvfactory.getCreatedObjects();	
	var invCircle = invCircleObj.item(0); // Input for intersect.
	
//	Rotated diskLineObj will pass through pointB, intersection with invCircle.	
//	Rotate diskLineObj by 180/p degrees.  p = sideCt.  Angle = ((PI/p) * (180/PI)).
	var rotatefactory1 = moi.command.createFactory( 'rotate' );
	rotatefactory1.setInput( 0, diskLineObj );
//	var FrameOrigin = centerPt; //(Frame has already been calculated.)
//	var Frame = moi.vectorMath.createFrame( FrameOrigin, FrameXAxis, FrameYAxis );
	rotatefactory1.setInput( 1, Frame );
	var rotateangle = 180 / sideCt;
	rotatefactory1.setInput( 2, rotateangle );
	rotatefactory1.setInput( 5, true );//Make copies.
	rotatefactory1.update();
	var diskLineBObj = rotatefactory1.getCreatedObjects();
	var diskLineB = diskLineBObj.item(0); // Input for intersect.
	
	var rotatefactory2 = moi.command.createFactory( 'rotate' );
	var rotateangle2 = -180 / sideCt;
	rotatefactory2.setInput( 0, diskLineObj );	
	rotatefactory2.setInput( 1, Frame );
	rotatefactory2.setInput( 2, rotateangle2 );
	rotatefactory2.setInput( 5, false ); // Do not make copies.
	rotatefactory2.update();
	var diskLineB2Obj = rotatefactory2.getCreatedObjects();
	var diskLineB2 = diskLineB2Obj.item(0); // Input for intersect.
//	moi.geometryDatabase.removeObjects(diskLineObj);
	
	// The intersect factory requires an object list which combines the two inputs.
	// Make object list of diskLineB and invCircle.
	var objlist1 = moi.geometryDatabase.createObjectList();
	objlist1.addObject( diskLineB );	
	var objlist3 = moi.geometryDatabase.createObjectList();	
	objlist3.addObject( invCircle );	
	// Combine objlist3 into objlist1
	for ( var i = 0; i < objlist3.length; ++i )
	{
		var obj = objlist3.item(i);
		objlist1.addObject( obj );
	}
	
	// Points ptB and ptB2 are the start and end points of the polygon side arc.
	// The intersect factory is asynchronous.	
	// ptB is generated using an intersect factory of a line and a circle. (similarly for ptB2)
	// (paraphrase of Michael, re chain script discussion and sweep asynchronous factory...)
	// An asynchronous factory requires that its inputs have an "object ID" set, by being added to the geometry database.
	// This was accomplished for diskLineB, diskLineB2, and invCircle, above, by using .update and .getCreatedObjects();,
	// for the two rotate factories, and the circle factory above.  (.item(0); is also often needed, but not always.) 
	// (Another way to get object ID set is with moi.geometryDatabase.addObjects( objectlist );)
	// (Creation just by .calculate(), just generates an object and does not add it into the geometry database automatically.)

	var intfactory1 = moi.command.createFactory( 'intersect' );
	intfactory1.setInput( 0, objlist1 );
	intfactory1.update();	
	var ptBobj = intfactory1.getCreatedObjects();
	// Turn ptBobj into a point.
	x = ptBobj.item(0).pt.x;
	y = ptBobj.item(0).pt.y;
	z = ptBobj.item(0).pt.z;
	var ptB = moi.vectorMath.createPoint( x, y, z );

	// Make an object list of diskLineB2 and invCircle.
	var objlist2 = moi.geometryDatabase.createObjectList();
	objlist2.addObject( diskLineB2 );	
	// Combine objlist3, (from above), into objlist2
	for ( var i = 0; i < objlist3.length; ++i )
	{
		var obj = objlist3.item(i);
		objlist2.addObject( obj );
	}

	var intfactory2 = moi.command.createFactory( 'intersect' );
	intfactory2.setInput( 0, objlist2 );
	intfactory2.update();
	var ptB2obj = intfactory2.getCreatedObjects();
//	var ptB2 = ptB2obj.item(0); //not used.

	// Turn ptB2obj into a point.
	x = ptB2obj.item(0).pt.x;
	y = ptB2obj.item(0).pt.y;
	z = ptB2obj.item(0).pt.z;
	var ptB2 = moi.vectorMath.createPoint( x, y, z );
	
	//	Create the arc with Frame center "centerInvPt", start PtB, end PtB2.
	var arcfactory = moi.command.createFactory( 'arccenter' );
	arcfactory.setInput( 0, Frame2 );
	arcfactory.setInput( 1, ptB );
	arcfactory.setInput( 2, ptB2 );
	arcfactory.update();
	var arcpolySideobj = arcfactory.getCreatedObjects(); //needed for arraycircular.
//	var arcpolySide = arcpolySideobj.item(0);//does not work for arraycircular.

// Do an array circular with arcpolySideobj, with sideCt copies.
// Note that "proxy" objects may be involved.  (See chainscript discussion help.)
	var arrayfactory = moi.command.createFactory( 'arraycircular' );
	arrayfactory.setInput( 0, arcpolySideobj );
	arrayfactory.setInput( 1, Frame );
	arrayfactory.setInput( 2, sideCt );
	arrayfactory.setInput( 3, 360.0 );
	arrayfactory.setInput( 4, 0.0 );
	arrayfactory.setInput( 5, 0.0 );
	arrayfactory.setInput( 6, 'fill' );
	arrayfactory.update();
	arrayfactory.calculate();
	var hyperPolyobj = arrayfactory.getCreatedObjects();

	// The basePoly may be subdivided as desired, and then inverted using Inversion3pt script.
	// Inversion3pt script may be used over and over with the generated polygons.	
	
//	Save each factory, (one curve or point each).
	disk.push( diskfactory );
	diskLine.push( diskLinefactory );
	circleInv.push( circleInvfactory );
	rotate1Line.push( rotatefactory1 );
	rotate2Line.push( rotatefactory2 );
	intersect1.push( intfactory1 );
	intersect2.push( intfactory2 );	
	arcSide.push( arcfactory );
	poly.push( arrayfactory );
}

function DoPoincareTessellation()
{
	var ui = moi.ui.commandUI;
	var vm = moi.vectorMath;

	//	Declare arrays for factory(s).
	var disk = new Array();
	var diskLine = new Array();
	var circleInv = new Array();
	var rotate1Line = new Array();
	var rotate2Line = new Array();
	var intersect1 = new Array();
	var intersect2 = new Array();
	var arcSide = new Array();
	var poly = new Array();
	
	// Get center of Poincaré disk of diameter 1 unit.
	var pointpicker = moi.ui.createPointPicker();
	if ( !GetPoint( pointpicker ) )
	return;
	var centerPt = pointpicker.pt;	

	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'PickPrompt' );
	moi.ui.showUI( 'OptionsPrompt' );
	moi.ui.showUI( 'Options' );
	moi.ui.endUIUpdate();		
	
	Update( centerPt, ui.sideCt.integerValue, ui.vertexCt.integerValue, ui.numpoints.integerValue, disk, diskLine, circleInv, rotate1Line, rotate2Line, intersect1, intersect2, arcSide, poly );

	var dialog = moi.ui.commandDialog;

	while ( 1 )
	{
		if ( !dialog.waitForEvent() )
		{
			//	Cancel factory arrays. (one curve or point each).
			Cancel( disk );
			Cancel( diskLine );
			Cancel( circleInv );
			Cancel( rotate1Line );
			Cancel( rotate2Line );
			Cancel( intersect1 );
			Cancel( intersect2 );
			Cancel( arcSide );
			Cancel( poly );
			return;
		}
			
		if ( dialog.event == 'done' )
			break;
			
		if ( dialog.event == 'sideCt' || dialog.event == 'vertexCt' || dialog.event == 'numpoints' )
		{

			//	Cancel factory arrays. (one curve or point each).
			Cancel( disk );
			Cancel( diskLine );
			Cancel( circleInv );
			Cancel( rotate1Line );
			Cancel( rotate2Line );
			Cancel( intersect1 );
			Cancel( intersect2 );
			Cancel( arcSide );
			Cancel( poly );
			
		Update( centerPt, ui.sideCt.integerValue, ui.vertexCt.integerValue, ui.numpoints.integerValue, disk, diskLine, circleInv, rotate1Line, rotate2Line, intersect1, intersect2, arcSide, poly );
		}
	}
//	Commit factory arrays. (one curve or point each).  (Do NOT need all of these.)
	Commit( disk );
//	Commit( diskLine );
//	Commit( circleInv );
	Commit( rotate1Line );
	Commit( rotate2Line );
//	Commit( intersect1 );
//	Commit( intersect2 );
	Commit( arcSide );
	Commit( poly);
}

DoPoincareTessellation();
