// config: norepeat
//	A MoI3D script to generate NACA four and five-digit series airfoils.
//  --------------------- An UNOFFICIAL MoI3D script --------------------------
//  AUTHOR - Hamish Mead
//  This is free and unencumbered software released into the public domain.
//  Anyone is free to copy, modify, publish, use, compile, sell, or
//  distribute this software, either in source code form or as a compiled
//  binary, for any purpose, commercial or non-commercial, and by any
//  means.
//  
//  In jurisdictions that recognize copyright laws, the author or authors
//  of this software dedicate any and all copyright interest in the
//  software to the public domain. We make this dedication for the benefit
//  of the public at large and to the detriment of our heirs and
//  successors. We intend this dedication to be an overt act of
//  relinquishment in perpetuity of all present and future rights to this
//  software under copyright law.
//  
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
//  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
//  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
//  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
//  OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
//  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
//  OTHER DEALINGS IN THE SOFTWARE.
//  
//  For more information, please refer to <http://unlicense.org>
//  ---------------------------------------------------------------------------
//  REVISION HISTORY
//  DATE        CHANGES
//  2015-02-21  Initial ALPHA release.
//  2015-03-23  See release notes.
//
//  RELEASE NOTES:
//  - Now generates all 'basic' NACA four-digit and five-digit series airfoils,
//    including 43012A.
//  - Variable thickness trailing edge (T.E) implemented.
//  - Zero T.E. thickness airfoils now draw correctly.
//  - Conformity of output with official NACA ordinates has been verified for
//    0012, 6518, 23012, 43012 & 43012A, see file:
//    'NACAAirfoilGenerator ordinates vs airfoils-2015-03-23.3dm'.
//
//  KNOWN LIMITATIONS:
//  - Five-digit airfoils with reflex camber lines not yet implemented.
//    CAUTIONS:
//    A4 coefficient (slider) value input can be set 'out of NACA limits'.
//  - Input parameter validation is rudimentary where it has been implemented.
//  - TODO: Implement four-digit modified, five-digit modified, and six-digit
//    series - http://m-selig.ae.illinois.edu/ads/naca980820.bas contains
//    working (public domain) code likely worth incorporating.
//
//
//  MANY THANKS:
//  To all the MoI3D script gurus for all their useful and inspiring script
//  examples and documentation, especially:
//  Michael Gibson, Brian McMillin (BEMFARMER), Max Smirnov (SMIRNOV),
//  Petr, David Morrill.
//
//  ---------------------------------------------------------------------------
//  MANY THANKS ALSO:
//  To the authors of these essential / invaluable documents and websites:
//
//  Original NACA/NASA Reports:
//  http://www.pdas.com/refs/rep460.pdf
//  http://www.pdas.com/refs/rep537.pdf
//  http://naca.central.cranfield.ac.uk/reports/1937/naca-report-610.pdf
//  http://naca.central.cranfield.ac.uk/reports/1945/naca-report-824.pdf
//  http://www.pdas.com/refs/tm4741.pdf
//  
//  Book:
//  Theory of Wing Sections - by Ira H. Abbott and Albert E. Von Doenhoff
//
//  Aeronautical information available online:
//  http://www.pdas.com/naca456.html - Ralph Carmichael
//  http://www.pdas.com/naca456refs.html - Ralph Carmichael
//  http://www.pdas.com/refs/aiaa5235.pdf - Ralph Carmichael
//  
//  http://www.dept.aoe.vt.edu/~mason/Mason_f/CAtxtTop.html - W. H. Mason
//  http://www.dept.aoe.vt.edu/~mason/Mason_f/CAtxtAppA.pdf - W. H. Mason
//
//  http://airfoiltools.com/airfoil/naca5digit - M. Lambden
//  http://airfoiltools.com/airfoil/naca4digit - M. Lambden
//
//  Other interesting documents:
//  - Generating NACA airfoils by novel ways:
//  "NACA FOUR-DIGIT AIRFOIL SECTION GENERATION USING CUBIC PARAMETRIC CURVE
//  SEGMENTS AND THE GOLDEN SECTION" -
//  https://ritdml.rit.edu/bitstream/handle/1850/11033/WScarbroughThesis04-28-1992.pdf
//  http://people.rit.edu/pnveme/bcurveindex.html
//
//   - Log aesthetic curves:
//  (For possible future use to describe thickness distribution of NACA 43012A)
//  http://www.yoshida-lab.net/english/research-e/log-aesthetic-curves/
//  http://www.yoshida-lab.net/PDF/laespacecurves.pdf
//  http://arxiv.org/ftp/arxiv/papers/1304/1304.7852.pdf
//  http://www.hindawi.com/journals/jam/2014/960302/
//  ---------------------------------------------------------------------------
//  DEFINITIONS OF VARIABLES
//
//  TODO: WORK IN PROGRESS
// 
//  As far as possible, variables names conform to those described in
//  NASA Technical Memorandum 4741 (TM4741):
//  'Computer Program To Obtain Ordinates for NACA Airfoils - December 1996
//  http://www.pdas.com/refs/tm4741.pdf
//
//  Those variable names definitions are copied here for ease of reference:
// 
//  A0,...,A4  - constants in airfoil equation
//  CHD, C     - airfoil chord
//  CLI        - design section lift coefficient
//  TOC        - thickness-chord ratio
//  XU, XL     - upper and lower surface locations of x
//  YU, YL     - upper and lower surface locations of y
// 
//  (DX         - basic selectable interval in profile generation
//               NOTE: DX not directly selectable by the user in this script)
//
//  NOTE: Comparing nomenclature for the four and five-digit airfoil
//  parameters from NACA TR460 and 537 with TM4741 can be a little confusing.
//  For ease of coding, this script takes a middle ground as detailed below:
//
//  Confusing four-digit airfoil parameter definitions:
//  NACA TR460:			m - "maximum ordinate"
//						p - "position of maximum ordinate"
//  -----------------------------------------------------------------
//  NASA TM4741:		p - "maximum ordinate of 2-digit camber line"
//						m - "chordwise location for maximum ordinate
//							of airfoil or camber line"
//
//							|	four-digit		|	five-digit
//  -------------------------------------------------------------------
//  NACA TR460/537:				m  p  xx		|	*			 m  k1	
//  NASA TM4741:			 	p  m  xx 		|	*			 r  k1 
//  airfoiltools.com:		 	M  P  XX 		|	L  P  Q  XX  r  k1 
//  										 		* Nomenclature not stated.
//
//  This script:
//    htm inputs:			 	M  P  XX 	 	|	L  P  Q  XX 
//    htm inputs assigned as: 	M  r  TOC  	 	|	L  P  Q  TOC  
//    Notes:
//		1. four-digit: By definition, P is equivalent to r.
//		2. five-digit: P resolves to r and k1 by switch statement.
//
//
//  ---------------------------------------------------------------------------
//  COEFFICIENTS A0,...,A4
//  NACA Four and Five-digit airfoil coefficients
//  According to NACA:
//  For an 'open' trailing edge, where at x = 1, y = 0.0021: set  A4 = -0.1015
//  For a 'closed' trailing edge, where at x = 1, y = 0:     set  A4 = -0.1036
//  NB: Intermediate values are also valid to vary thickness of trailing edge.
//  
//  ---------------------------------------------------------------------------

// Global variables
var ui = moi.ui.commandUI;
var Boundary = moi.geometryDatabase.createObjectList();
var FactoryTD43012A = moi.command.createFactory( 'interpcurve' );
var ArrayTD43012A =
		// XY points were extracted using reverse NACA transformation script:
		// n2td.js
		// from the curve object:
		// "0005-004-001-006-joined blend A4:-0.1015"
		// in the file:
		// "002 - deriving NACA 43012A T.D. using 'reverse NACA transformation' script.3dm"
		// Points form a best fit curve corresponding very closely to NACA43012
        // L.E. radius, exactly with NACA43012 (A4 = -0.1015) T.E. AFT of
        // 0.421250946 CHD and two further blended curves matching lower points
        // of NACA43012A in the area where NACA43012 is otherwise under-cambered.
		//
		// For ultimate smoothness, an additional (G3: Bulge=1) blend could be
		// created to match NACA 43012 T.E for any value of A4, however the
        // small improvement in curve smoothness achieved is not considered to
        // warrant the effort - or added code complexity.
		[[0,0],
		[0.0001096582626,0.001866288578],
		[0.0004385849506,0.003700497175],
		[0.0009866357859,0.005521595836],
		[0.001753570375,0.007337920199],
		[0.002739052316,0.009148904697],
		[0.003942649343,0.01095797741],
		[0.005363833519,0.01277468278],
		[0.007001981465,0.01461351008],
		[0.008856374636,0.01648811149],
		[0.01092619963,0.01840281295],
		[0.01321054856,0.02035231783],
		[0.01570841944,0.02233014677],
		[0.0184187166,0.0243299383],
		[0.02134025123,0.02634330184],
		[0.02447174185,0.02836048118],
		[0.02781181488,0.03037125906],
		[0.03135900525,0.03236550962],
		[0.03511175706,0.03433354311],
		[0.03906842421,0.03626632429],
		[0.04322727118,0.03815560452],
		[0.04758647377,0.03999398924],
		[0.05214411988,0.04177495451],
		[0.05689821038,0.04349282308],
		[0.06184665998,0.04514270894],
		[0.06698729811,0.04672043933],
		[0.07231786992,0.04822246315],
		[0.07783603725,0.04964575517],
		[0.08353937964,0.05098772517],
		[0.08942539543,0.05224614109],
		[0.09549150281,0.05341907427],
		[0.101735041,0.054504874],
		[0.1081532713,0.05550217657],
		[0.1147433786,0.0564099526],
		[0.1215024722,0.05722759341],
		[0.1284275873,0.05795503518],
		[0.1355156863,0.05859291589],
		[0.1427636602,0.05914275705],
		[0.1501683297,0.05960715802],
		[0.157726447,0.05998998704],
		[0.1654346968,0.0602965498],
		[0.173289698,0.06053371595],
		[0.1812880051,0.06071000243],
		[0.1894261099,0.06083568749],
		[0.1977004426,0.06092311515],
		[0.2061073739,0.06098680065],
		[0.2146432162,0.061034292],
		[0.2233042254,0.06106534736],
		[0.2320866025,0.06107918838],
		[0.2409864953,0.06107510418],
		[0.25,0.06105246255],
		[0.2591231629,0.06101071474],
		[0.2683519824,0.06094939302],
		[0.2776824104,0.06086810076],
		[0.2871103542,0.06076649532],
		[0.2966316785,0.06064426404],
		[0.3062422068,0.06050109485],
		[0.3159377237,0.06033664289],
		[0.3257139763,0.06015049566],
		[0.3355666766,0.05994213936],
		[0.3454915028,0.05971092992],
		[0.3554841015,0.05945607246],
		[0.3655400897,0.0591766134],
		[0.3756550564,0.05887144994],
		[0.3858245649,0.05853936162],
		[0.3960441546,0.05817906917],
		[0.4063093427,0.05778932572],
		[0.4166156266,0.05736904564],
		[0.4269584857,0.05691748126],
		[0.4373333832,0.05643472991],
		[0.4477357684,0.0559217294],
		[0.4581610783,0.05537949846],
		[0.4686047402,0.05480906823],
		[0.4790621731,0.0542114799],
		[0.4895287901,0.05358778255],
		[0.5,0.05293903098]];

// calc yt using four or five-digit series airfoil thickness distribution curve
function chordwise_thickness_yt( A, TOC, x, A4 )
{	
	// A4 user settable by slider:
	// For an 'open' trailing edge:  A4 = -0.1015
    // For a 'closed' trailing edge: A4 = -0.1036
	// TODO: Consider defining T.E. thickness by user point selection.
	var A0 = 0.2969, A1 = -0.1260, A2 = -0.3516, A3 = 0.2843;

	// Equivalent best-fit coefficients for the 43012A Thickness Distribution
	// by (polynomial) expression.
    // Notes:
	// - 'Run 83': Derived by manipulation of a regression using free Desmos.com
	// - LE radius slightly less than 43012!
	// - Aft 50% has slight oscillation.
	// - Coefficient for close state not yet determined.
	/*
	var B0 =    0.268177851026;
	var B1 =    0.259896957907;
	var B2 =    -1.54468824791;
	var B3 =    -13.4021148969;
	var B4 =    116.581507912;
	var B5 =    -384.436624913;
	var B6 =    661.974564496;
	var B7 =    -613.006883954;
	var B8 =    348.90849238;
	var B9 =    -418.493965901;
	var B10 =    764.241346323;
	var B11 =    -755.436402127;
	var B12 =    362.865463717;
	var B13 =    -68.7766745913;
	*/
	
	// Standard thickness distribution for four and five digit series airfoils
	if( A == 0 )
	{
		yth = TOC/0.2 * ((A0 * Math.pow(x, 0.5)) + (A1 * x) + 
			(A2 * Math.pow(x, 2)) + (A3 * Math.pow(x, 3)) + 
			(A4 * Math.pow(x, 4)));
	}
	else if ( A == 1 ) // 43012A Only!
	{
		/*
        // Simpler, ideal solution but not accurate enough:
        // TODO: Investigate other curve expressions (Log Aesthetic, etc)
		yth = TOC/0.2 * ((B0 * Math.pow(x, 0.5)) + (B1 * x) + 
			(B2 * Math.pow(x, 2)) + (B3 * Math.pow(x, 3)) + 
			(B4 * Math.pow(x, 4)) + (B5 * Math.pow(x, 5)) + 
			(B6 * Math.pow(x, 6)) + (B7 * Math.pow(x, 7)) + 
			(B8 * Math.pow(x, 8)) + (B9 * Math.pow(x, 9)) + 
			(B10 * Math.pow(x, 10)) + (B11 * Math.pow(x, 11)) + 
			(B12 * Math.pow(x, 12)) + (B13 * Math.pow(x, 13)));
		*/
	
		if ( x == 0)
		{
			yth = 0;
		}
		else if ( x > 0 && x < 0.42125 ) // 0.42125 for A4 = -0.1015
		{
			//draw line from (x, 0) to (x, 0.1)
			var YtFactory = moi.command.createFactory( "line" );
			YtFactory.setInput( 0, moi.vectorMath.createPoint( x, 0, 0 ) );
			YtFactory.setInput( 1, moi.vectorMath.createPoint( x, 0.001, 0 ) );
			
			//extend line to FactoryTD43012A curve
			var ObjectsToExtend = moi.geometryDatabase.createObjectList();
			ObjectsToExtend.addObject( YtFactory.calculate().item(0) );
			// NB: 43012A Thickness Distribution curve (Boundary object)
			// is added during script initialisation.
			
			// No need to commit ObjectsToExtend before they're extended,
			// so next line comment out
			//moi.geometryDatabase.addobjects(ObjectsToExtend);

			var YTExtendFactory = moi.command.createFactory( "extend" );
			YTExtendFactory.setinput(0, ObjectsToExtend);
			YTExtendFactory.setinput(1, Boundary);
			// No need to see the extended lines, so next line comment out
			//YTExtendFactory.commit();
			
			//get length (corresponds directly to get y value) of extended line
			yth = YTExtendFactory.calculate().item(0).getLength();
			
			// Clean up for the next cycle
			YTExtendFactory.reset();
			YtFactory.reset();

		}
		else if ( x >= 0.42125 ) // 0.42125 for A4 = -0.1015
		{
			yth = TOC/0.2 * ((A0 * Math.pow(x, 0.5)) + (A1 * x) + 
			(A2 * Math.pow(x, 2)) + (A3 * Math.pow(x, 3)) + 
			(A4 * Math.pow(x, 4)));
		}
	}

	// Return the airfoil thickness value at x chord-wise position.
    return yth;
}

// Calculate airfoil points, mean camber line and single airfoil curve
function CalculateAirfoil()
{	
	var CHD = ui.chord_length.numericValue;
	var num_stations = ui.num_stations.numericValue;
	var cosine_spacing =  ui.cosine_spacing.value;
	var plot_mean_chord =  ui.plot_mean_chord.value;
	var A4 = ui.coeff_A4.numericValue;
	
	var PointObjectListUpper = moi.geometryDatabase.createObjectList();
	var PointObjectListLower = moi.geometryDatabase.createObjectList();
	var FactoryAirfoil =  moi.command.createFactory( 'interpcurve' );
	var FactoryAirfoilMeanChord = moi.command.createFactory( 'interpcurve' );
    var AirfoilCornerPoint = moi.command.createFactory( 'addpoint' );
	var PointFactory = moi.command.createFactory( 'point' );
	
	var dyc_dx, g, k1, L, M, x, XU, YU, XL, YL, P, Q, r, TOC, theta, yc, yt, yth;
	var A = 0;
	var i = 0;
	var z = 0;
    var OpenTE = 0;
	
	// Assign variables for the series user selected
	var SeriesSelected = ui.SeriesSelect.value;
	switch ( SeriesSelected )
	{
	case 'four':
		// Basic four-digit vars
		M = ui.four_M.numericValue/100;
		r = ui.four_P.numericValue/10;
		TOC = ui.four_XX.numericValue/100;
		break;
	case 'five':
		// Basic five-digit vars
		L = ui.five_L.numericValue;
		P = ui.five_P.integerValue;
		Q = ui.five_Q.integerValue;
		TOC = ui.five_XX.numericValue/100;
		A = 0;
		break;
	case 'fiveA':
		// For 43012A ONLY

		// To enable generation of the uncambered 43012A symmetric form,
		// i.e. Thickness Distribution, uncomment the following four vars:
		
		L = ui.five_L.numericValue;
		P = ui.five_P.integerValue;
		Q = ui.five_Q.integerValue;
		TOC = ui.five_XX.numericValue/100;
		
        /*
		// ...and comment out the next FOUR:
		L = ui.fiveA_L.numericValue;
		P = ui.fiveA_P.integerValue;
		Q = ui.fiveA_Q.integerValue;
		TOC = ui.fiveA_XX.numericValue/100;
		*/
        A = 1;
        break;
	}
	// Five-digit series factors 'L'
	// 'P' is used as a table lookup.
	if ( SeriesSelected == 'five' || 'fiveA' )
	{
		// CAMBER DESIGNATOR: L
		// L = 2 represents 1 * 3/20 = 0.15 design coefficient of lift
		// L = 4 represents 2 * 3/20 = 0.3 design coefficient of lift
		// L = 6 represents 3 * 3/20 = 0.45 design coefficient of lift
		var CLI = L / 2;

		// MEAN CAMBER CONSTANTS: r, k1
		//
		// mld: Mean Line Designation
		// p: Position of maximum camber
		//  
		// TODO: Investigate Mason's CAtxtAppA.pdf (link above).
		// Page A-6 suggests slightly more accurate values for r and k1, below.
		// initial experiments with those values give questionable results.
		// mld   p       r       k1
		// 210  0.05  0.0580  361.400
		// 220  0.10  0.1260   51.640
		// 230  0.15  0.2025   15.957
		// 240  0.20  0.2900    6.643
		// 250  0.25  0.3910    3.230
		switch( P )
		{	
			case 0: // 000 mean line - not strictly valid,
					// but enables showing that four and five-digit
					// symmetric airfoils are identical.
				r = 0.0;
				k1 =0.0;
				break;
			case 1: //210 mean line
				r = 0.0580;
				k1 = 361.400;
				break;
			case 2: //220 mean line
				r = 0.1260;
				k1 = 51.640;
				break;
			case 3: //230 mean line
				r = 0.2025;
				k1 = 15.957;
				break;
			case 4: //240 mean line
				r = 0.2900;
				k1 =  6.643;
				break;
			case 5: //250 mean line
				r = 0.3910;
				k1 = 3.230;
				break;
			default:
				if ( P > 5 )
				{
					moi.ui.alert("Hello test pilot!\n\nValid values for P (five-digit airfoil) are: (0), 1, 2, 3, 4 or 5.");
					r = 0.3910;
					k1 = 3.230;
				}
		}
	}

	// Main calculation loop, further subdivided in two sections: FWD and AFT.
	for (i = 0; i < num_stations + 1; ++i)
	{
		if (cosine_spacing == 0)
		{
			x = i/num_stations;
		}
		// TODO: Consider offering user half-cosine spacing -
		// cosine spacing FWD of position of max camber,
        // regular spacing AFT of that point.
		else if (cosine_spacing == 1)
		{
			x = (1 - Math.cos((i/num_stations) * Math.PI))/2;
		}

		// FWD mean line, upper and lower profile ordinates, from: 0 <= x <= r
		if (x < r )
		{	
			if ( SeriesSelected == 'four')
			{
				// FWD four-digit mean line
				yc = ( M / Math.pow( r, 2 )) * (( 2 * r * x ) - Math.pow( x, 2 ));
				// FWD four-digit gradient
				dyc_dx = ( 2 * M / Math.pow( r, 2 )) * ( r - x );
			}
			else if ( SeriesSelected == 'five' || 'fiveA')
			{
				// FWD five-digit mean line
				yc = ( CLI *( k1 / 6 ) * (( Math.pow( x, 3 )) - ( 3 * r * Math.pow( x, 2 )) + ( Math.pow( r, 2 ) * ( 3 - r ) * x )));
				// FWD five-digit gradient
				dyc_dx = ( CLI * ( k1 / 6 ) * (( 3 * Math.pow( x, 2 )) - ( 6 * r * x ) + ( Math.pow( r, 2 ) * ( 3 - r ))));
			}
			
			// Airfoil thickness at x
			yt = chordwise_thickness_yt( A, TOC, x, A4 );
			// Slope of mean chord at x
			theta = Math.atan( dyc_dx );
			
			// FWD Upper profile ordinates
			XU = ( x - yt * Math.sin(theta));
			YU = ( yc + yt * Math.cos(theta));
			// FWD Lower profile ordinates
			XL = ( x + yt * Math.sin(theta));
			YL = ( yc - yt * Math.cos(theta));
			
			// Scale FWD coordinates to user specified chord and create a list object
			PointFactory.setInput( 0, moi.vectorMath.createPoint( XU * CHD, YU * CHD, z  ) );
			PointObjectListUpper.addObject( PointFactory.calculate().item(0) );
			PointFactory.reset();
			
			PointFactory.setInput( 0, moi.vectorMath.createPoint( XL * CHD, YL * CHD, z  ) );
			PointObjectListLower.addObject( PointFactory.calculate().item(0) );
			PointFactory.reset();
			
			if (plot_mean_chord == 1)
			{
				FactoryAirfoilMeanChord.createInput( 'point' );
				FactoryAirfoilMeanChord.setInput( i, moi.vectorMath.createPoint( x * CHD, yc * CHD, z  ) );
			}
		}
		
		// AFT mean line, upper and lower ordinates profile,
		// from: r <= x <= 1
		else if( x >= r )
		{	
			if ( SeriesSelected == 'four')
			{
				// AFT four-digit mean line
				yc = ( M / Math.pow(( 1 - r ), 2 )) * (( 1 - 2 * r ) + ( 2 * r * x ) - Math.pow( x, 2 ));
				// AFT four-digit gradient
				dyc_dx = ( 2 * M / Math.pow(( 1 - r ), 2 )) * ( r - x );
			}
			else if ( SeriesSelected == 'five' || 'fiveA')
			{
				// AFT five-digit mean line
				yc = CLI * ( k1 / 6 ) * Math.pow( r, 3 ) * ( 1 - x );
				// AFT five-digit gradient
				dyc_dx = ( CLI * ( -k1 / 6 ) * Math.pow( r, 3 ) );
			}
			
			// Airfoil thickness at x			
			yt = chordwise_thickness_yt( A, TOC, x, A4 );
			// Slope of mean chord at x
			theta = Math.atan(dyc_dx);
			
			// AFT Upper profile ordinates
			XU = x - yt * Math.sin(theta);
			YU = yc + yt * Math.cos(theta);
			
			// AFT Lower profile ordinates
			XL = x + yt * Math.sin(theta);
			YL = yc - yt * Math.cos(theta);
			
			// Scale AFT coordinates to user specified chord and create an ObjectList of points
			PointFactory.setInput( 0, moi.vectorMath.createPoint( XU * CHD, YU * CHD, z  ) );
			PointObjectListUpper.addObject( PointFactory.calculate().item(0) );
			PointFactory.reset();
			
			PointFactory.setInput( 0, moi.vectorMath.createPoint( XL * CHD, YL * CHD, z  ) );
			PointObjectListLower.addObject( PointFactory.calculate().item(0) );
			PointFactory.reset();
			
			if ( plot_mean_chord == 1 )
			{
				FactoryAirfoilMeanChord.createInput( 'point' );
				FactoryAirfoilMeanChord.setInput( i, moi.vectorMath.createPoint( x * CHD, yc * CHD, z  ) );
			}
		}
	}
    
    // Transfer upper and lower airfoil curve points to one interpcurve 'Through
    // Points' curve factory:
	// Step 1: copy REVERSE order of PointObjectListUpper into FactoryAirfoil
    // (Start at the airfoil T.E. and proceed forward over the airfoil
	// upper profile to the L.E.)
    for ( g = 0; g < num_stations + 1; ++g )
	{
		FactoryAirfoil.createInput( 'point' );
		FactoryAirfoil.setInput( g, PointObjectListUpper.item(num_stations - g).pt.get );
	}
	
	// Step 2: copy NORMAL order of PointObjectListLower into FactoryAirfoil
    // Continue along the airfoil lower profile from L.E. to T.E - note 
    // 0'th point not added as it would duplicate last point from Step 1.
	for ( g = 1; g < num_stations + 1; ++g )
	{
		FactoryAirfoil.createInput( 'point' );
		FactoryAirfoil.setInput( num_stations + g, PointObjectListLower.item(g).pt.get );
	}
    
    // Make last point a corner point, regardless whether it's needed
    // (to make the cusp of a closed T.E.) or not.
    FactoryAirfoil.createInput( 'bool' );
	FactoryAirfoil.setInput( 2 * num_stations + 1, true );
    
    // Create a line segment between the ends of the open airfoil curve	
    var FactoryLineCloseTE = moi.command.createFactory( 'line' );
    
    //If airfoil is closed
    if (moi.vectorMath.pointsWithinTolerance(
    PointObjectListUpper.item( num_stations ).pt.get,
    PointObjectListLower.item( num_stations ).pt.get ))
    {
        FactoryLineCloseTE.cancel();
        OpenTE = 0;
    }
    else
    {
        // If airfoil is open at T.E., create a line segment between curve ends.
        FactoryLineCloseTE.setInput( 0, PointObjectListUpper.item( num_stations ).pt.get );
        FactoryLineCloseTE.setInput( 1, PointObjectListLower.item( num_stations ).pt.get );
        FactoryLineCloseTE.update();
        OpenTE = 1;
    }
	
    FactoryAirfoil.update();
    
    if (ui.plot_mean_chord.value == 1)
    {
        FactoryAirfoilMeanChord.update();
    }
	
	PointFactory.cancel();
	// TODO: Remove if points output not required
	//PointObjectListUpper.deleteAll();
	//PointObjectListLower.deleteAll();
	
	return {'OpenTE': OpenTE, 
			'FactoryLineCloseTE': FactoryLineCloseTE,
			'FactoryAirfoil': FactoryAirfoil, 
			'PointObjectListLower': PointObjectListLower, 
			'PointObjectListUpper': PointObjectListUpper, 
			'FactoryAirfoilMeanChord' : FactoryAirfoilMeanChord};
}


function GenerateAirfoil()
{	
	/*
	TODO: Possibly enable drawing of airfoils in other than XY plane.
	var pointpicker = moi.ui.createPointPicker();
	if ( !GetPoint( pointpicker ) )
		return;
	
	var cplane = pointpicker.ptframe;
	*/
    
    // Create 43012A Thickness Distribution Through Point curve
	for (var q = 0; q < (ArrayTD43012A.length); q++ )
	{
		FactoryTD43012A.createInput( 'point' );
		FactoryTD43012A.setInput( q, moi.vectorMath.createPoint(
            ArrayTD43012A[q][0], ArrayTD43012A[q][1], 0 ) );
	}
	
	// Use 43012A Thickness Distribution as an Extend boundary
    // Done here, so as it's only required once
	Boundary.addObject( FactoryTD43012A.calculate().item(0) );
	// Uncomment to display
    // moi.geometryDatabase.addobjects(Boundary);
	
	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'PickPrompt' );
	moi.ui.hideUI( 'CancelContainer' );
	moi.ui.showUI( 'SeriesPrompt' );
	
	var result = CalculateAirfoil();
	//var PointsUpper = result.PointObjectListUpper;
	//var PointLower = result.PointObjectListLower;
	var factoryAF = result.FactoryAirfoil;
	var factoryMC = result.FactoryAirfoilMeanChord;
	var factoryTE = result.FactoryLineCloseTE;
	var AirfoilOpen = result.OpenTE;

	// Wait for any changes to the UI or cancel or done.
	var dlg = moi.ui.commandDialog;
	while ( 1 )
	{
		switch ( ui.SeriesSelect.value )
		{
		case 'four':
			moi.ui.showUI( 'FourSeries' );
			moi.ui.showUI( 'TrailingEdgeOptions' );
			moi.ui.hideUI( 'FiveSeries' );
			moi.ui.hideUI( 'FiveSeries43012A' );
			break;
		case 'five':
			moi.ui.showUI( 'FiveSeries' );
			moi.ui.showUI( 'TrailingEdgeOptions' );
			moi.ui.hideUI( 'FourSeries' );
			moi.ui.hideUI( 'FiveSeries43012A' );
			break;
		case 'fiveA':
			moi.ui.showUI( 'FiveSeries43012A' );
			moi.ui.showUI( 'TrailingEdgeOptions' );
			moi.ui.hideUI( 'FourSeries' );
			moi.ui.hideUI( 'FiveSeries' );
			break;
		}

		moi.ui.showUI( 'AirfoilOptions' );
		moi.ui.showUI( 'DoneCancelContainer' );
		moi.ui.endUIUpdate();

		if ( !dlg.waitForEvent() )
		{
			factoryAF.cancel();
            factoryMC.cancel();
			factoryTE.cancel();
			return false;
		}

		if ( dlg.event == 'done' )
			break;
		
		if (dlg.event == 'SeriesSelect' || dlg.event == 'four_P' ||
		dlg.event == 'four_M' || dlg.event == 'four_XX' ||
		dlg.event == 'five_L' || dlg.event == 'five_P' || 
		dlg.event == 'five_XX' || dlg.event == 'ftzot_A' ||
		dlg.event == 'coeff_A4' || dlg.event == 'chord_length' ||
		dlg.event == 'num_stations' || dlg.event == 'cosine_spacing' ||
		dlg.event == 'plot_mean_chord')
		{
			// Enable refresh
			factoryAF.cancel();
			factoryMC.cancel();
			factoryTE.cancel();
			AirfoilOpen = 0;
			
			// TODO: Consider enabling user to select 'generate points and/or profile'
			//PointsUpper.cancel();
			//PointsLower.cancel();

			result = CalculateAirfoil();
			factoryAF = result.FactoryAirfoil;
            factoryTE = result.FactoryLineCloseTE;
            AirfoilOpen = result.OpenTE;
            
            if ( ui.plot_mean_chord.value == 1 )
            {
                factoryMC = result.FactoryAirfoilMeanChord;
            }
		}
	}
	
	// TODO: Consider enabling user to select 'generate points and/or profile'
	//factoryAF.commit();
	//factoryTE.commit();
    
    // Join the T.E. line and the airfoil curve
	if ( AirfoilOpen == 1 )
	{
        var ObjectsToJoin = moi.geometryDatabase.createObjectList();
        ObjectsToJoin.addObject(factoryAF.calculate().item(0));
        ObjectsToJoin.addObject(factoryTE.calculate().item(0));
        
        moi.geometryDatabase.addobjects(ObjectsToJoin);
        var FactoryJoin = moi.command.createFactory( 'join' );
        FactoryJoin.setInput( 0, ObjectsToJoin );
        FactoryJoin.commit();
        
		factoryAF.cancel();
        factoryTE.cancel();
	}
    else
    {   
        factoryAF.commit();
    }

    if ( ui.plot_mean_chord.value == 1 )
    {
        factoryMC.commit();
    }
	// TODO: Consider enabling user to select 'generate points and/or profile'
	//moi.geometryDatabase.addObjects(PointsUpper);
	//moi.geometryDatabase.addObjects(PointsLower);
}

 GenerateAirfoil();
