// Script to graph Nielsen's spiral in the CAD program Moment of Inspiration, MoI. By Brian McMillin, June, 2014.
// Nielsen's spiral is a 2d curve defined by X = si(t), where si(t) = sine integral Si(t) - PI/2,
// and Y = ci(t) = Ci(t), where the cosine integral ci(t) is calculated from Cin(t).
// See the attached pdf notes on the Nielsen's spiral.

function alert( msg )
{
	moi.ui.commandUI.alert( msg );
}

function Cancel( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].cancel();
	factories.length = 0;
}

function Commit( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].commit();
}

function evalNielsen(aScale, siY, ciY)
{
	var x = aScale * ciY;
	var y = aScale * siY;
	var z = 0.0;
	return moi.vectorMath.createPoint( x, y, z );
}

function Update( aScale, tLength, numIntervals, nValue, Sifactories, sifactories, Cinfactories, cifactories, Nielsenfactories )
{
	Cancel( Sifactories );
	Cancel( sifactories );
	Cancel( Cinfactories );
	Cancel( cifactories );
	Cancel( Nielsenfactories );
	
	var Sifactory = moi.command.createFactory( 'interpcurve' );
	var sifactory = moi.command.createFactory( 'interpcurve' );
	var Cinfactory = moi.command.createFactory( 'interpcurve' );
	var cifactory = moi.command.createFactory( 'interpcurve' );
	var Nielsenfactory = moi.command.createFactory( 'interpcurve' );

	var z = 0.0;
	tLength = Math.abs(tLength);
	numIntervals = Math.abs(numIntervals);
	var gamma = 0.57721566490153286; //Euler-Mascheroni constant.
	nValue = 10;  // nValue has been set to = 10, nothing else.
	var totalIntervals = tLength * numIntervals;
	
//	aScale changes the scale of Spiral.
//	tLength is the maximum abscissa (max x value,) of the Si, si, Cin and ci curves.
//	numIntervals is the number of intervals (or points) per unit tLength.
//	The Si, si and Cin curves have one additional point at x=0.
//	The initial point for Si is (0,0,0).
//	The initial point for si is (0,-PI/2,0).
//	The initial point for Cin is (0,0,0).     
//	The ci curve is undefined at x=0.  The first ci curve point is evaluated at x = 1/numIntervals.
//	The first point of the Nielsen's spiral is evaluated at 1/numIntervals.
//	nValue is the order of the Gauss-Legendre Quadrature integration.  n = 10.
//  The weights and roots are hard coded in an array.

//	Add a point at origin.
	var pt = moi.vectorMath.createPoint( 0, 0, 0 );
	var halfPI = Math.PI /2;
	var pt2 = moi.vectorMath.createPoint( 0, -halfPI, 0 );
	Sifactory.createInput( 'point' );
	Sifactory.setInput( Sifactory.numInputs - 1, pt );
	sifactory.createInput( 'point' );
	sifactory.setInput( sifactory.numInputs - 1, pt2 );	
	Cinfactory.createInput( 'point' );
	Cinfactory.setInput( Cinfactory.numInputs - 1, pt );

// Legendre-Gauss quadrature, for order n = 6:
//var weights6 = [ 0.3607615730481386, 0.3607615730481386, 0.4679139345726910, 0.4679139345726910, 0.1713244923791704, 0.1713244923791704 ];
//var roots6 = [ 0.6612093864662645, -0.6612093864662645, -0.2386191860831969, 0.2386191860831969, -0.9324695142031521,0.9324695142031521 ];

// Legendre-Gauss quadrature, for order n = 10:
var weights = [ 0.2955242247147529,  0.2955242247147529 , 0.2692667193099963, 0.2692667193099963, 0.2190863625159820, 0.2190863625159820, 0.1494513491505806, 0.1494513491505806, 0.0666713443086881, 0.0666713443086881 ];
var roots = [ -0.1488743389816312, 0.1488743389816312,  -0.4333953941292472 ,  0.4333953941292472, -0.6794095682990244,  0.6794095682990244, -0.8650633666889845, 0.8650633666889845, -0.9739065285171717, 0.9739065285171717 ];
	
//	Initialize areas for Si(t) and Cin(t) curves. 
	var sumSi = 0.0;
	var sumCin = 0.0;
	var sigmaS = 0.0;
	var sigmaC = 0.0;

//alert ("weights[0] = " + weights[0] + "\nweights[1] = " + weights[1] + "\nroots[0] = " + roots[0] + "\nroots[1] = " + roots[1]);
	
// a and b are the lower and upper limits of integration.
	var a = 0.0; // a1 is zero.
	var b = 1 / numIntervals;
	var ba = (b - a) /2;
//alert( "ba =" + ba);	
	for ( var i = 1; i <= totalIntervals; ++i )	
	{
		var b = i / numIntervals;
		var ab = (a + b)/2;
		
		for ( var j = 0; j < nValue; ++j )
		{	
			sigmaS += weights[j] * Math.sin(ba * roots[j] + ab) / (ba * roots[j] + ab);  
//alert("sigmaS =" + sigmaS);
			sigmaC += weights[j] * ( 1 - Math.cos(ba * roots[j] + ab)) / (ba * roots[j] + ab); 
		}	
//alert("sigmaC =" + sigmaC);			
//		sumSi  += ba * sigmaS; // This does NOT work.
		sumSi = ba * sigmaS;  // This does work.
		
//		sumCin += ba * sigmaC; // This does NOT work.
		sumCin = ba * sigmaC;  // This does work.		
//alert("sumSi =" + sumSi + "\nsumCin =" + sumCin );

		var ptSi = moi.vectorMath.createPoint( b, sumSi, z );
		var siY = sumSi - halfPI;
		var ptsi = moi.vectorMath.createPoint( b, siY, z );
		var ptCin = moi.vectorMath.createPoint( b, sumCin, z );
		var ciY = gamma + Math.log(b) - sumCin;
		var ptci = moi.vectorMath.createPoint( b, ciY, z ); 
		var ptN = evalNielsen(aScale, siY, ciY)
		//Add point to the interpcurve factory.
		Sifactory.createInput( 'point' );
		Sifactory.setInput( Sifactory.numInputs - 1, ptSi );
	
		sifactory.createInput( 'point' );
		sifactory.setInput( sifactory.numInputs - 1, ptsi );

		Cinfactory.createInput( 'point' );
		Cinfactory.setInput( Cinfactory.numInputs - 1, ptCin );

		cifactory.createInput( 'point' );
		cifactory.setInput( cifactory.numInputs - 1, ptci );

		Nielsenfactory.createInput( 'point' );
		Nielsenfactory.setInput( Nielsenfactory.numInputs - 1, ptN );		
		
		a = b; // next a is previous b.
	}
	Sifactories.push( Sifactory );
	// The next codeline displays curves as they are created.
	Sifactory.update();
	sifactories.push( sifactory );
	Cinfactories.push( Cinfactory );
	cifactories.push( cifactory );
	Nielsenfactories.push( Nielsenfactory );
	
//  Each integration interval, of width = 1/numIntervals, is evaluated with nValue weights and arguments = abscissae or roots).
//  And for each succeeding interval, the area is added to the previous areas, (NO DOES NOT WORK THAT WAY,) and another curve point is generated.
//	The Si(t) curve and the Cin(t) curve must be integrated.
//	SI - PI/2 yields si, and Cin must be converted to ci, to get the Nielsen's Spiral.
//  An (x,y,z) point on the Nielsen's Spiral is (ci(t), si(t), 0).

	for ( var i = 0; i < Sifactories.length; ++i )
		{
			Sifactories[i].update();
		}	

	for ( var i = 0; i < sifactories.length; ++i )
		{
			sifactories[i].update();
		}

	for ( var i = 0; i < Cinfactories.length; ++i )
		{
			Cinfactories[i].update();
		}		
		
	for ( var i = 0; i < cifactories.length; ++i )
		{
			cifactories[i].update();
		}		
		
	for ( var i = 0; i < Nielsenfactories.length; ++i )
		{
			Nielsenfactories[i].update();
		}		
}

function NielsensSpiral()
{
	var Sifactories = new Array();
	var sifactories = new Array();
	var Cinfactories = new Array();
	var cifactories = new Array();
	var Nielsenfactories = new Array();
	var UI = moi.ui.commandUI; //Abbreviation	
	
	Update( UI.aScale.numericValue, UI.tLength.numericValue, UI.numIntervals.integerValue, UI.nValue.integerValue, Sifactories, sifactories, Cinfactories, cifactories, Nielsenfactories )

	var dlg = moi.ui.commandDialog;
	
	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
		{
			Cancel( Sifactories );
			Cancel( sifactories );			
			Cancel( Cinfactories );			
			Cancel( cifactories );			
			Cancel( Nielsenfactories );			
			return;
		}
			
		if ( dlg.event == 'done' )
			break;
			
		if ( dlg.event == 'aScale' || dlg.event == 'tLength' || dlg.event == 'numIntervals' || dlg.event == 'nValue'  )
		{
Update( UI.aScale.numericValue, UI.tLength.numericValue, UI.numIntervals.integerValue, UI.nValue.integerValue, Sifactories, sifactories, Cinfactories, cifactories, Nielsenfactories )
		}
	}
	Commit( Sifactories );
	Commit( sifactories );
	Commit( Cinfactories );	
	Commit( cifactories );
	Commit( Nielsenfactories );
}

NielsensSpiral();