// By bemfarmer, October 5, 2013
// An Oloid has a developable surface, and is used in mixers, and as a toy.
// The Oloid script creates many line segments of equal length of sqrt(3), between the two circular arcs,
// one set from theta = 0 to theta = PI/2, and another similar set, on the other side.
// This avoids the parameterization singularity for t = +/- 2*PI/3.
// Loft the line segments to create 1/4 of the surface of the Oloid.  
// Mirror twice about the origin to create the rest of the surface.
// Boolean union the surfaces to create the solid Oloid.
// Note that the position of the APoints on the first curve, and the BPoints on the second
// curve are "involutive" related by trigonometry, so points on the second curve cannot be uniformly
// distributed.  So many line segments are needed for loft to create the Oloid.
// Lineweb and Sweep would not create the Oloid shape. (IMHO)
// http://www.heldermann-verlag.de/jgg/jgg01_05/jgg0113.pdf

// config: norepeat

#include "GetObjects.js"

function alert( msg )
{
	moi.ui.commandUI.alert( msg );
}

function Cancel( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].cancel();
		
	factories.length = 0;
}

function Commit( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].commit();
}

function EvalAPoint( t )
{
	// The length of each line segment is sqrt(3).
	// For better line spacing, rather than run theta from 0 to 2*PI/3,
	// theta is run from 0 to PI/2, and again on the other side from 0 to PI/2, with 
	// the y coordinate formula negated, and the x and z formulas swapped,
	// which is why A2Point and B2Point are used in addition to A1Point and B1Point.
	var theta;
	theta = ((Math.PI * 1.0) / 2.0) * t;
//	theta = ((Math.PI * 2.0) / 3.0) * t; //This range not used.
	var x = Math.sin( theta );
	var y = -0.5 - Math.cos( theta );
	var z = 0.0;
	
	return moi.vectorMath.createPoint( x, y, z );
}

function EvalBPoint( t )
{
	var theta;
	theta = ((Math.PI * 1.0) / 2.0) * t;
	cos_theta = Math.cos( theta );
	var x = 0.0;
	var y = 0.5 - (cos_theta / (1 + cos_theta ));
	var z = Math.sqrt(1 + 2*cos_theta) / (1 + cos_theta );
	
	return moi.vectorMath.createPoint( x, y, z );
}

function EvalA2Point( t )
{
	var theta;
	theta = ((Math.PI * 1.0) / 2.0) * t;
	var x = 0.0;
	var y = 0.5 + Math.cos( theta );
	var z = Math.sin( theta );
	
	return moi.vectorMath.createPoint( x, y, z );
}

function EvalB2Point( t )
{
	var theta;
	theta = ((Math.PI * 1.0) / 2.0) * t;
	cos_theta = Math.cos( theta );
	var x = Math.sqrt(1 + 2*cos_theta) / (1 + cos_theta );
	var y = -0.5 + (cos_theta / (1 + cos_theta ));
	var z = 0.0;
	
	return moi.vectorMath.createPoint( x, y, z );
}

function Update( numpoints, factories )
{
	Cancel( factories );
//	numpoints = (numpoints * 2) + 1; //Negative theta values not used.

	// Build individual lines.
	// t is a fraction, and 0 <= t <= 1	
	for ( var i = 0; i < (numpoints); ++i )
	{
		var t = i / (numpoints-1);
//		var t = (i - (numpoints-1)/2) / ((numpoints - 1)/2); //Negative theta values not used.
		var Apt = EvalAPoint( t );
		var Bpt = EvalBPoint( t );
		var factory = moi.command.createFactory( 'line' );
		factory.setInput( 0, Apt );
		factory.setInput( 1, Bpt );
		factories.push( factory );
	}

	// Now do the other quadrant with y negated, and swapped x and z formulas.
	// This amounts to a 180 degree rotation and a 90 degree rotation.
	// 1 is subtracted from numpoints i index value to avoid duplicate line.
	for ( var i = 0; i < (numpoints-1); ++i )
	{
		var t = i / (numpoints-1);
		var A2pt = EvalA2Point( t );
		var B2pt = EvalB2Point( t );
		var factory = moi.command.createFactory( 'line' );
		factory.setInput( 0, A2pt );
		factory.setInput( 1, B2pt );
		factories.push( factory );
	}
	
	for ( var i = 0; i < factories.length; ++i )
		factories[i].update();
}

function DoOloid()
{
	var factories = new Array();
	Update( moi.ui.commandUI.numpoints.integerValue, factories );
	
	// Wait for cancel or done.	
	var dlg = moi.ui.commandDialog;
	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
		{
			Cancel( factories );
			return;
		}
			
		if ( dlg.event == 'done' )
			break;
			
		if ( dlg.event == 'numpoints' )
			Update( moi.ui.commandUI.numpoints.integerValue, factories );
	}
	
	Commit( factories );
}

DoOloid();
